@@ -14,6 +14,7 @@ from pathlib import Path
 import json
 import time
 import datetime
+import h5py
 
 import numpy as np
 
@@ -26,13 +27,25 @@ def get_file_sz(FileName):
 
 
 def get_file_creation_date(file_name):
-    return os.path.getctime(file_name)
+    if file_name:
+        return os.path.getctime(file_name)
 
 
 def get_file_creation_date_str(file_name):
-    return time.strftime(
-        "%Y-%m-%d",
-        time.gmtime(get_file_creation_date(file_name)))
+    date = get_file_creation_date(file_name)
+    if date:
+        return time.strftime("%Y-%m-%d", time.gmtime(date))
+
+
+def get_file_mod_date(file_name):
+    if file_name:
+        return os.path.getmtime(file_name)
+
+
+def get_file_mod_date_str(file_name):
+    date = get_file_mod_date(file_name)
+    if date:
+        return time.strftime("%Y-%m-%d", time.gmtime(date))
 
 
 def parse_csv_header(file_name, key):
@@ -82,6 +95,109 @@ def json_multiget(data, key_arg, default_val=None):
         return data.get(key_arg, default_val)
 
 
+def list_zi_h5_traces(file_name=None):
+    """List traces in a Zurich Instruments H5 file."""
+    file_name = get_zi_h5_file_name(file_name)
+
+    data_file = h5py.File(file_name, 'r')
+    keys = list(data_file.keys())
+    data_names = []
+    print("Data file '{:s}' contents: ".format(Path(file_name).stem))
+    for key_ind, key in enumerate(keys):
+        header_names = data_file.get(
+            key + '/dev1940/demods/0/sample/chunkheader')[()].dtype.names
+        name_ind = [ind for ind, name in enumerate(header_names)
+                    if name == 'name'][0]
+        data_name = data_file.get(
+            key + '/dev1940/demods/0/sample/chunkheader')[0][name_ind].decode()
+        print(key_ind, data_name)
+        data_names.append(data_name)
+
+    return data_names
+
+
+def get_zi_h5_file_name(file_name=None, fail_on_no_file=True):
+    """Find a Zurich Instruments .h5 file in the current dir."""
+    if file_name is None:
+        file_names = list_files_with_extension('.', ext='h5')
+        if len(file_names) == 0:
+            raise ValueError("No .h5 files found")
+        elif len(file_names) == 1:
+            file_name = file_names[0]
+        else:
+            print("Multiple .h5 file founds, this program only supports a "
+                  "single file per folder")
+            file_name = file_names[0]
+
+    if fail_on_no_file and not file_name:
+        raise ValueError("No file name given")
+
+    return file_name
+
+
+def get_zi_h5_trace(file_name=None, trace_name=None, trace_index=None):
+    """Get trace data from a Zurich Instruments .h5 file"""
+    file_name = get_zi_h5_file_name(file_name)
+    data_file = h5py.File(file_name, 'r')
+    keys = list(data_file.keys())
+
+    data_names = list_zi_h5_traces(file_name)
+
+    if not trace_index:
+        for ind, data_name in enumerate(data_names):
+            if data_name == trace_name:
+                trace_index = ind
+                break
+
+        if trace_index is None:
+            raise RuntimeError("Trace '{:}' not found".format(trace_name))
+
+    key = keys[trace_index]
+
+    farr = np.array(data_file.get(key + '/dev1940/demods/0/sample/frequency'))
+    varr = np.array(data_file.get(key + '/dev1940/demods/0/sample/r'))
+    parr_dbm = 10*np.log10(varr**2/50*1E3)
+    rbwarr = np.array(data_file.get('000/dev1940/demods/0/sample/bandwidth'))
+    if (np.diff(rbwarr[np.logical_not(np.isnan(rbwarr))]) != 0).any():
+        print("WARNING: data has variable bandwidth")
+
+    return {'farr': farr, 'parr_dbm': parr_dbm, 'varr': varr,
+            'rbw': np.mean(rbwarr), 'rbwarr': rbwarr}
+
+
+def extract_zi_h5_traces(file_name=None):
+    """Extract traces from Zurich Instruments H5 files"""
+    file_name = get_zi_h5_file_name(file_name)
+
+    data_file = h5py.File(file_name, 'r')
+    keys = list(data_file.keys())
+
+    data_names = list_zi_h5_traces(file_name)
+
+    for key_ind, key in enumerate(keys):
+        data_name = data_names[key_ind]
+        print("Exporting " + data_name)
+        farr = np.array(data_file.get(
+            key + '/dev1940/demods/0/sample/frequency'))
+        varr = np.array(data_file.get(
+            key + '/dev1940/demods/0/sample/r'))
+        parr_dbm = 10*np.log10(varr**2/50*1E3)
+        bw = np.array(data_file.get('000/dev1940/demods/0/sample/bandwidth'))
+        if (np.diff(bw[np.logical_not(np.isnan(bw))]) != 0).any():
+            print("WARNING: data has variable bandwidth")
+        file_name = '{:d}_{:s}.dat'.format(
+            key_ind, data_name.replace(':', '_'))
+        header_str = \
+            'HF2LI sweeper data, index: {:d}, name: {:s}, bw: {:.0f}'.format(
+                key_ind, data_name, np.nanmean(bw))
+        header_str += '\nFrequency (Hz), Amplitude (dbm)'
+        np.savetxt(file_name, np.transpose([farr, parr_dbm]), delimiter=',',
+                   header=header_str)
+
+    print("\nAll done.")
+    input("Press any key to continue...")
+
+
 def check_file_exists(file_path):
     """Check if a file exists."""
     try:
@@ -476,14 +592,16 @@ def read_beam_steering_log(dir_list=None):
                 if Path(Path(file_name).parts[0]).suffix == '.zip':
                     zip_file_name = Path(Path(file_name).parts[0])
                     archive = zipfile.ZipFile(zip_file_name, mode='r')
-                    file = io.BufferedReader(archive.open(Path(*Path(file_name).parts[1:]).as_posix(), mode='r'))
+                    file = io.BufferedReader(
+                        archive.open(Path(
+                            *Path(file_name).parts[1:]).as_posix(), mode='r'))
                 else:
                     file = open(file_name)
 
                 pos_log_data = np.loadtxt(
                     file, delimiter=',', usecols=[0, 2],
                     dtype={'names': ('date', 'pos'),
-                            'formats': (datetime.datetime, 'float')})
+                           'formats': (datetime.datetime, 'float')})
 
                 if pos_log[ind].get('t0') is None:
                     pos_log[ind]['t0'] = datetime.datetime.strptime(
@@ -492,9 +610,9 @@ def read_beam_steering_log(dir_list=None):
                 pos_log[ind]['tarr'] = np.append(
                     pos_log[ind]['tarr'],
                     np.array([(datetime.datetime.strptime(entry[0],
-                                "%Y-%m-%d %H:%M:%S.%f")
-                                - pos_log[ind]['t0']).total_seconds()/60/60
-                                for entry in pos_log_data]))
+                              "%Y-%m-%d %H:%M:%S.%f")
+                               - pos_log[ind]['t0']).total_seconds()/60/60
+                              for entry in pos_log_data]))
 
                 pos_log[ind]['val'] = np.append(
                     pos_log[ind]['val'],
@@ -504,7 +622,9 @@ def read_beam_steering_log(dir_list=None):
                 print("Failed to read log file with exception", excpt)
                 print("Retrying line-by-line")
                 if Path(Path(file_name).parts[0]).suffix == '.zip':
-                    file = io.BufferedReader(archive.open(Path(*Path(file_name).parts[1:]).as_posix(), mode='r'))
+                    file = io.BufferedReader(
+                        archive.open(Path(
+                            *Path(file_name).parts[1:]).as_posix(), mode='r'))
                 else:
                     file = open(file_name)
 
@@ -536,7 +656,7 @@ def read_beam_steering_log(dir_list=None):
                     pos_log[ind]['tarr'] = np.append(
                         pos_log[ind]['tarr'],
                         float((data_ts -
-                                pos_log[ind]['t0']).total_seconds()/60/60))
+                               pos_log[ind]['t0']).total_seconds()/60/60))
 
                     pos_log[ind]['val'] = np.append(
                         pos_log[ind]['val'],
