@@ -335,7 +335,7 @@ def imshow_ex(
         img, ax=None,
         vmin=None, vmax=None, vrng_algorithm='minmax', min_vspan=None,
         auto_vrng_percentile_min=0, auto_vrng_percentile_max=99.5,
-        bad_color=no_signal_color, logscale=False, cmap='virids',
+        bad_color=no_signal_color, logscale=False, cmap='viridis',
         title_str=None, with_hist=True, is_angle=False):
     """Show an image with a histogram and colorbar.
 
@@ -356,7 +356,7 @@ def imshow_ex(
         img2[invmask] = np.log10(img[invmask])
         img = img2
 
-    if vrng_algorithm not in ['minmax', 'perctentile']:
+    if vrng_algorithm not in ['minmax', 'percentile']:
         print("Unknown vrng algorithm, resetting to 'minmax")
         vrng_algorithm = 'minmax'
 
@@ -626,6 +626,10 @@ def hide_xticklabels():
     """Hide X tick labels but keep the ticks."""
     plt.gca().axes.get_xaxis().set_ticklabels([])
 
+def hide_yticklabels():
+    """Hide Y tick labels but keep the ticks."""
+    plt.gca().axes.get_yaxis().set_ticklabels([])
+
 
 def set_ticks_inside():
     plt.gca().tick_params(axis='both', which='both', direction='in')
